import json

import click

from autumn8.cli import options
from autumn8.cli.cli_environment import CliEnvironment
from autumn8.common.config.settings import CloudServiceProvider
from autumn8.common.config.supported_instances import find_instance_config
from autumn8.common.types import DEFAULT_CLOUD_ZONES_CONFIG
from autumn8.lib import logging
from autumn8.lib.api import cloud

logger = logging.getLogger(__name__)


@click.group()
def cloud_commands_group():
    pass


@cloud_commands_group.command()
@options.use_environment
@options.use_organization_id
@options.use_quiet_mode
@options.use_cloud_provider_picker(optional=True)
@click.option(
    "-m",
    "--model_id",
    help="Model ID to get the deployments for",
    prompt_required=False,
    default=None,
)
def list_deployments(
    organization_id,
    model_id,
    environment: CliEnvironment,
    cloud_provider,
    quiet,
):
    """List running deployments."""
    logger.info("Fetching the list of deployments...")
    deployments = cloud.get_running_deployments(
        organization_id,
        environment,
        model_id=model_id,
        service_provider=cloud_provider,
    )

    click.echo(json.dumps(deployments, indent=4))
    return


@cloud_commands_group.command()
@click.option(
    "-hw",
    "-t",
    "--machine_type",
    prompt="Machine type (ie. c5.2xlarge)",
    type=str,
    help="Server type to use for the deployment",
    # TODO: add a better interactive prompt listing all available servers
)
@options.use_environment
@options.use_organization_id
@options.use_quiet_mode
@click.option(
    "-m",
    "--model_id",
    "--model-id",
    prompt=True,
    type=int,
    help="Model ID to deploy",
    # TODO: add a better interactive prompt listing all available models
)
@options.use_cloud_provider_picker(optional=False)
def deploy(
    organization_id: int,
    model_id: int,
    machine_type: str,
    environment: CliEnvironment,
    cloud_provider: CloudServiceProvider,
    quiet,
):
    """Deploy a model from AutoDL onto cloud."""
    instance_config = find_instance_config(
        machine_type, fetch_data_from_cloud_info=False
    )
    real_hardware_provider = instance_config.service_provider
    logger.info(
        "Launching a new deployment onto %s...",
        DEFAULT_CLOUD_ZONES_CONFIG[real_hardware_provider],
    )
    deployments = cloud.deploy(
        organization_id,
        environment,
        machine_type=machine_type,
        service_provider=cloud_provider,
        model_id=model_id,
    )

    click.echo(json.dumps(deployments, indent=4))


@cloud_commands_group.command()
@options.use_environment
@options.use_organization_id
@options.use_quiet_mode
@options.use_cloud_provider_picker(optional=False)
@click.option(
    "-d",
    "--deployment_id",
    prompt=True,
    help="ID of the deployment to terminate",
)
def terminate_deployment(
    organization_id,
    deployment_id,
    environment: CliEnvironment,
    cloud_provider,
    quiet,
):
    """Terminate a running deployment."""
    response = cloud.terminate_deployment(
        organization_id, environment, deployment_id, cloud_provider
    )
    click.echo(json.dumps(response, indent=4))



from typing import Dict

from autumn8.common.config.settings import CloudServiceProvider

ZoneConfig = Dict[CloudServiceProvider, str]

DEFAULT_CLOUD_ZONES_CONFIG: ZoneConfig = {
    CloudServiceProvider.AMAZON: "us-east-1",
    CloudServiceProvider.GOOGLE: "us-central1",
    CloudServiceProvider.ORACLE: "us-sanjose-1",
    CloudServiceProvider.AZURE: "useast",
}



import sys
import time

import click
import questionary
from questionary import Choice

from autumn8.cli import options, pending_uploads
from autumn8.cli.commands import cloud, models
from autumn8.cli.interactive import (
    announce_model_upload_response,
    fetch_user_data,
)
from autumn8.common._version import __version__
from autumn8.lib.service import resume_upload_model

# from autumn8.cli.cli_environment import CliEnvironment
# from autumn8.common.config.s3 import init_s3_client

# environment = CliEnvironment.STAGING
# s3 = init_s3_client(environment.value.s3_host)
# s3_bucket_name = environment.value.s3_bucket_name
# mpus = s3.list_multipart_uploads(Bucket=s3_bucket_name, MaxUploads=200)

# print("mpus", len(mpus["Uploads"]))
# for mpu in mpus["Uploads"]:
#     print(mpu["Key"])

try:
    current_pending_uploads = pending_uploads.retrieve_pending_uploads()
except (ValueError, AttributeError):
    # data format stored is incompatible with the current version
    # hope that the pending upload will get cleaned up by the S3 lifecycle policy
    # TODO: add the S3 lifecycle policy for that, lol - https://docs.aws.amazon.com/AmazonS3/latest/userguide/mpu-abort-incomplete-mpu-lifecycle-config.html
    pending_uploads.forget_all_pending_uploads()
    current_pending_uploads = {}

# TODO: save costs by configuring https://docs.aws.amazon.com/AmazonS3/latest/userguide/mpu-abort-incomplete-mpu-lifecycle-config.html
# TODO: detect if the cached upload is still available on S3; forget it if it got cleaned up
for key in current_pending_uploads:
    resume_args = current_pending_uploads[key]
    print("You have pending upload of {}".format(resume_args["model_file"]))
    continue_upload = questionary.select(
        "Do you want to continue upload?",
        choices=[
            Choice(title="Yes", value="Y"),
            Choice(title="No", value="n"),
            Choice(title="Drop upload", value="drop"),
        ],
        use_shortcuts=True,
    ).unsafe_ask()

    if continue_upload == "" or continue_upload == "Y":
        model_upload_response = resume_upload_model(
            {**resume_args, **resume_args["kwargs"]}
        )
        announce_model_upload_response(model_upload_response)
        sys.exit(0)

    if continue_upload == "drop":
        pending_uploads.abort_and_forget_upload(resume_args["run_id"])


@options.use_environment
def test_connection(environment):
    """
    Test AutoDL connection with the current API key.
    Displays the user's email address upon successful connection.
    """
    user_data = fetch_user_data(environment)
    print(f"Hello! You're authenticated as {user_data['email']}")


@click.group()
@click.version_option(version=__version__)
def main():
    pass


main.command()(test_connection)

all_commands = []

for command in [
    *models.model_commands_group.commands.values(),
    *cloud.cloud_commands_group.commands.values(),
]:
    main.add_command(command)


if __name__ == "__main__":
    main()
